###################################################################################################
# zmsproxyobject.py
#
# $Id: zmsproxyobject.py,v 1.7 2004/11/23 23:04:47 dnordmann Exp $
# $Name:  $
# $Author: dnordmann $
# $Revision: 1.7 $
#
# Implementation of class ZMSProxyObject (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Product Imports.
from zmsobject import ZMSObject
from zmscontainerobject import ZMSContainerObject
import _globals


################################################################################################
################################################################################################
###   
###   C l a s s
###   
################################################################################################
################################################################################################

class ZMSProxyObject(ZMSContainerObject):


    # ------------------------------------------------------------------------------------------
    #  ZMSProxyObject.__init__: 
    # ------------------------------------------------------------------------------------------
    def __init__(self, base, url_base, proxy, recursive=False):
      """ Constructor """
      if _globals.debug( self):
        _globals.writeLog( self, "[__init__]")
      self.base = base
      self.url_base = url_base
      self.id = proxy.id
      self.meta_type = proxy.meta_type
      self.dGlobalAttrs = proxy.dGlobalAttrs
      self.REQUEST = proxy.REQUEST
      self.proxy = proxy
      self.recursive = recursive


    # ---------------------------------------------------------------------------------------------
    #  ZMSLinkElement.__proxy__:
    # ---------------------------------------------------------------------------------------------
    def __proxy__(self):
      if _globals.debug( self):
        _globals.writeLog( self, "[__proxy__]")
      return self


    # ------------------------------------------------------------------------------------------
    #  ZMSProxyObject.__request__ 
    # ------------------------------------------------------------------------------------------
    def __request__(self, REQUEST):
      proxy = self.proxy
      if type( REQUEST) is type( ''):
        return proxy.REQUEST
      return REQUEST


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.absolute_url
    # ---------------------------------------------------------------------------------------------
    def absolute_url(self):
      url_base = self.url_base
      rtn = url_base
      return rtn


    # ------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getMetaobjIds:
    # ------------------------------------------------------------------------------------------
    def getMetaobjIds(self, sort=1):
      proxy = self.proxy
      rtn = proxy.getMetaobjIds( sort)
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.filtered_child_actions:
    # ---------------------------------------------------------------------------------------------
    def filtered_child_actions(self, REQUEST):
      proxy = self.proxy
      req = self.__request__( REQUEST)
      rtn = proxy.filtered_child_actions( req)
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.filtered_insert_actions:
    # ---------------------------------------------------------------------------------------------
    def filtered_insert_actions(self, path, REQUEST):
      proxy = self.proxy
      req = self.__request__( REQUEST)
      rtn = proxy.filtered_insert_actions( path, req)
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getConfProperty
    # ---------------------------------------------------------------------------------------------
    def getConfProperty(self, key, default=None):
      proxy = self.proxy
      rtn = proxy.getConfProperty( key, default)
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getObjAttrs
    # ---------------------------------------------------------------------------------------------
    def getObjAttrs(self, meta_type=None):
      proxy = self.proxy
      rtn = proxy.getObjAttrs( meta_type)
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.getObjAttrValue:
    # ---------------------------------------------------------------------------------------------
    def getObjAttrValue(self, obj_attr, REQUEST):
      if _globals.debug( self):
        _globals.writeLog( self, "[getObjAttrValue]")
      proxy = self.proxy
      req = self.__request__( REQUEST)
      rtn = proxy.getObjAttrValue( obj_attr, req)
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getDocumentElemen
    # ---------------------------------------------------------------------------------------------
    def getDocumentElement(self):
      if _globals.debug( self):
        _globals.writeLog( self, "[getDocumentElement]")
      base = self.base
      rtn = base.getDocumentElement( )
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getDCCoverage
    # ---------------------------------------------------------------------------------------------
    def getDCCoverage(self, REQUEST={}):
      proxy = self.proxy
      rtn = proxy.getDCCoverage( REQUEST)
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getDCDescription
    # ---------------------------------------------------------------------------------------------
    def getDCDescription(self, REQUEST):
      proxy = self.proxy
      rtn = proxy.getDCDescription( REQUEST)
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getPageExt
    # ---------------------------------------------------------------------------------------------
    def getPageExt(self, REQUEST):
      proxy = self.proxy
      # print "getPageExt", self, proxy
      req = self.__request__( REQUEST)
      rtn = proxy.getPageExt( req)
      return rtn


    # ------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getTitle
    # ------------------------------------------------------------------------------------------
    def getTitle(self, REQUEST):
      if _globals.debug( self):
        _globals.writeLog( self, "[getTitle]")
      proxy = self.proxy
      req = self.__request__( REQUEST)
      rtn = proxy.getTitle( req)
      return rtn


    # ------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getTitlealt:
    # ------------------------------------------------------------------------------------------
    def getTitlealt(self, REQUEST):
      if _globals.debug( self):
        _globals.writeLog( self, "[getTitlealt]")
      proxy = self.proxy
      req = self.__request__( REQUEST)
      rtn = proxy.getTitlealt( req)
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.filteredChildNodes:
    # ---------------------------------------------------------------------------------------------
    def filteredChildNodes(self, REQUEST={}, meta_types=None, management_interface=False):
      rtn = []
      recursive = self.recursive
      if recursive:
        proxy = self.proxy
        req = self.__request__( REQUEST)
        if hasattr( proxy, 'filteredChildNodesPROXY'):
          rtn = map( lambda x: ZMSProxyObject( self, self.absolute_url()+'/'+x.id, x, recursive), proxy.filteredChildNodesPROXY( proxy, req, meta_types, management_interface))
        else:
          rtn = map( lambda x: ZMSProxyObject( self, self.absolute_url()+'/'+x.id, x, recursive), proxy.filteredChildNodes( req, meta_types, management_interface))
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getTeaserElements:
    # ---------------------------------------------------------------------------------------------
    def getTeaserElements(self, REQUEST, current=None):
      if _globals.debug( self):
        _globals.writeLog( self, "[getTeaserElements]")
      rtn = []
      recursive = self.recursive
      if recursive:
        proxy = self.proxy
        req = self.__request__( REQUEST)
        if hasattr( proxy, 'getTeaserElementsPROXY'):
          rtn = proxy.getTeaserElementsPROXY( proxy, req, current)
        else:
          rtn = proxy.getTeaserElements( req, current)
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getChildNodes:
    # ---------------------------------------------------------------------------------------------
    def getChildNodes(self, REQUEST={}, meta_types=None):
      if _globals.debug( self):
        _globals.writeLog( self, "[getChildNodes]")
      rtn = []
      recursive = self.recursive
      if recursive:
        proxy = self.proxy
        req = self.__request__( REQUEST)
        if hasattr( proxy, 'getChildNodesPROXY'):
          rtn = map( lambda x: ZMSProxyObject( self, self.absolute_url()+'/'+x.id, x, recursive), proxy.getChildNodesPROXY( proxy, req, meta_types))
        else:
          rtn = map( lambda x: ZMSProxyObject( self, self.absolute_url()+'/'+x.id, x, recursive), proxy.getChildNodes( req, meta_types))
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getLevelnfc
    # ---------------------------------------------------------------------------------------------
    def getLevelnfc(self, REQUEST):
      if _globals.debug( self):
        _globals.writeLog( self, "[getLevelnfc]")
      proxy = self.proxy
      rtn = proxy.getLevelnfc( REQUEST)
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getLevel
    # ---------------------------------------------------------------------------------------------
    def getLevel(self):
      if _globals.debug( self):
        _globals.writeLog( self, "[getLevel]")
      base = self.base
      rtn = base.getLevel( ) + 1
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getObjProperty
    # ---------------------------------------------------------------------------------------------
    def getObjProperty(self, key, REQUEST={}, default=None):
      proxy = self.proxy
      req = self.__request__( REQUEST)
      if hasattr( proxy, 'getObjPropertyPROXY'):
        rtn = proxy.getObjPropertyPROXY( proxy, key, req, default)
      else:
        rtn = proxy.getObjProperty( key, req, default)
      return rtn


    # -------------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getParentNode
    # -------------------------------------------------------------------------------------------------
    def getParentNode(self):
      rtn = self.base
      return rtn


    # -------------------------------------------------------------------------------------------------
    #  ZMSProxyObject.isActive
    # -------------------------------------------------------------------------------------------------
    def isActive(self, REQUEST):
      proxy = self.proxy
      rtn = proxy.isActive( REQUEST)
      return rtn


    # -------------------------------------------------------------------------------------------------
    #  ZMSProxyObject.isResource:
    # -------------------------------------------------------------------------------------------------
    def isResource(self, REQUEST):
      proxy = self.proxy
      req = self.__request__( REQUEST)
      rtn = proxy.isResource( req)
      return rtn


    # ------------------------------------------------------------------------------------------
    #  ZMSProxyObject.isPage:
    # ------------------------------------------------------------------------------------------
    def isVisible(self):
      proxy = self.proxy
      rtn = proxy.isPage( )
      return rtn


    # ------------------------------------------------------------------------------------------
    #  ZMSProxyObject.isPageElement:
    # ------------------------------------------------------------------------------------------
    def isPageElement(self):
      proxy = self.proxy
      rtn = proxy.isPageElement( )
      return rtn


    # ------------------------------------------------------------------------------------------
    #  ZMSProxyObject.isVisible:
    # ------------------------------------------------------------------------------------------
    def isVisible(self, REQUEST):
      proxy = self.proxy
      req = self.__request__( REQUEST)
      rtn = proxy.isVisible( req)
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject._getBodyContent:
    # ---------------------------------------------------------------------------------------------
    def _getBodyContent(self, REQUEST):
      proxy = self.proxy
      req = self.__request__( REQUEST)
      rtn = proxy._getBodyContent( req)
      return rtn
    
    
    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getBodyContent:
    # ---------------------------------------------------------------------------------------------
    def getBodyContent(self, REQUEST):
      proxy = self.proxy
      req = self.__request__( REQUEST)
      rtn = proxy.getBodyContent( req)
      return rtn

    
    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.display_icon:
    # ---------------------------------------------------------------------------------------------
    def display_icon(self, REQUEST, meta_type=''): 
      proxy = self.proxy
      req = self.__request__( REQUEST)
      rtn = proxy.display_icon( req, meta_type)
      return rtn
    
    
    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getLangStr:
    # ---------------------------------------------------------------------------------------------
    def getLangStr(self, key, lang=None):
      proxy = self.proxy
      rtn = proxy.getLangStr( key, lang)
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getMetadictAttrs:
    # ---------------------------------------------------------------------------------------------
    def getMetadictAttrs(self, meta_type=None):
      proxy = self.proxy
      rtn = proxy.getMetadictAttrs( meta_type)
      return rtn
    

    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getMetadictAttr:
    # ---------------------------------------------------------------------------------------------
    def getMetadictAttr(self, key):
      proxy = self.proxy
      rtn = proxy.getMetadictAttr( key)
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.hasAccess:
    # ---------------------------------------------------------------------------------------------
    def hasAccess(self, REQUEST):
      proxy = self.proxy
      req = self.__request__( REQUEST)
      rtn = proxy.hasAccess( req)
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getObjVersion:
    # ---------------------------------------------------------------------------------------------
    def getObjVersion(self, REQUEST={}):
      proxy = self.proxy
      req = self.__request__( REQUEST)
      rtn = proxy.getObjVersion( req)
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.breadcrumbs_obj_path:
    # ---------------------------------------------------------------------------------------------
    def breadcrumbs_obj_path(self):
      ob = self.getParentNode()
      if hasattr( ob, 'breadcrumbs_obj_pathPROXY'):
        rtn = ob.breadcrumbs_obj_pathPROXY( )
      else:
        rtn = ob.breadcrumbs_obj_path( )
      rtn.append( self)
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getDeclUrl:
    # ---------------------------------------------------------------------------------------------
    def getDeclUrl(self, REQUEST={}):
      if self.getConfProperty('ZMS.pathhandler',0) == 0:
        rtn = self.absolute_url()
      else:
        base = self.base
        req = self.__request__( REQUEST)
        rtn = base.getDeclUrl( req)
        rtn += '/' + self.getDeclId( req)
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  ZMSProxyObject.getHref2IndexHtml:
    # ---------------------------------------------------------------------------------------------
    def getHref2IndexHtml(self, REQUEST, deep=1): 
      proxy = self.proxy
      rtn = ZMSObject.getHref2IndexHtml( self, REQUEST, deep)
      return rtn


###################################################################################################
